import { Request, Response, NextFunction } from "express";
import * as jwt from "jsonwebtoken";
import { JWT_SECRET } from "../config/contants";
import { TypedRequest, USER_TYPE } from "../config/types";
import Auth from "../models/Auth";

export const authMiddleware = (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const token = req.header("auth_token");

  if (!token) {
    return res.status(401).send("Authorization denied");
  } else {
    const decodedToken = <jwt.JwtPayload>jwt.verify(token, JWT_SECRET);
    if (decodedToken) {
      req.query.userId = decodedToken.userId;
      return next();
    } else {
      return res.status(401).send("Unauthorized Access");
    }
  }
};

export const organizationMiddleware = async (
  req: TypedRequest<{ userId: string }, any>,
  res: Response,
  next: NextFunction
) => {
  try {
    const org = await Auth.findOne({ userId: req.query.userId });
    if (org && org.userType === USER_TYPE.ORGANIZATION) {
      return next();
    } else {
      throw new Error("Organization not found");
    }
  } catch (error) {
    return res.status(400).send(error);
  }
};

export const candidateMiddleware = async (
  req: TypedRequest<{ userId: string }, any>,
  res: Response,
  next: NextFunction
) => {
  try {
    const org = await Auth.findOne({ userId: req.query.userId });
    if (org && org.userType === USER_TYPE.CANDIDATE) {
      return next();
    } else {
      throw new Error("Candidate not found");
    }
  } catch (error) {
    return res.status(400).send(error);
  }
};
