import cv2
import dlib
import imutils
import numpy as np
from imutils import face_utils
from scipy.spatial import distance as dist
from scripts.parameters import EYE_AR_THRESH, EYE_AR_CONSEC_FRAMES


def eye_aspect_ratio(eye):
    A = dist.euclidean(eye[1], eye[5])
    B = dist.euclidean(eye[2], eye[4])
    C = dist.euclidean(eye[0], eye[3])
    ear = (A + B) / (2.0 * C)
    return ear

 
detector = dlib.get_frontal_face_detector()
predictor = dlib.shape_predictor("models/shape_predictor_68_face_landmarks.dat")


(lStart, lEnd) = face_utils.FACIAL_LANDMARKS_IDXS["left_eye"]
(rStart, rEnd) = face_utils.FACIAL_LANDMARKS_IDXS["right_eye"]

def count_blinks(filename, start=0, end=0): 
    COUNTER = 0
    TOTAL = 0
    vs = cv2.VideoCapture(filename)
    fps = vs.get(cv2.CAP_PROP_FPS)

    start_frame = start * fps
    end_frame = end * fps
    current_frame = start_frame

    vs.set(cv2.CAP_PROP_POS_FRAMES, start_frame)

    while (current_frame<=end_frame):
        ret, frame = vs.read()

        if ret == True:
            frame = imutils.resize(frame, width=800)
            gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
            rects = detector(gray, 0)
            current_frame = current_frame + 1
            print('FRAME : '+str(current_frame) + ' Blinks :'+ str(TOTAL))

            for rect in rects:

                shape = predictor(gray, rect)
                shape = face_utils.shape_to_np(shape)

                leftEye = shape[lStart:lEnd]
                rightEye = shape[rStart:rEnd]
                leftEAR = eye_aspect_ratio(leftEye)
                rightEAR = eye_aspect_ratio(rightEye)

                ear = (leftEAR + rightEAR) / 2.0 

                if ear < EYE_AR_THRESH:
                    COUNTER += 1
                else:
                    if COUNTER >= EYE_AR_CONSEC_FRAMES:
                        TOTAL += 1
                    COUNTER = 0
        else:
            break
                
    vs.release()
    return TOTAL